import numpy
import math
import random
from curtin_options_tools import helper

def getSigma(stock, strike, price, time, riskfree=0.03, dividend=0):
    fnew = 1
    a = 0
    b = 1
    count = 0
    while (fnew > 0.0001) or (fnew < -0.0001):
        c = (b+a)/2
        fnew = BlackScholes(stock, strike, c, time, riskfree, dividend) - price
        if fnew > 0:
            b = c
        else:
            a = c
        count = count + 1
        
        if count > 100:
            print("Volatility = 0.00. Poor price data!")
            raise SigmaZeroError
    return c

class SigmaZeroError(BaseException):
    pass

"""def secant(price):
    oldx = 0
    newx = 1
    fnew = 1
    while (fnew > 0.001) or (fnew < -0.001):
        fnew = BlackScholes(newx) - price
        fold = BlackScholes(oldx) - price
        temp = newx
        newx = newx - fnew*(newx - oldx)/(fnew - fold)
        oldx = temp
    return newx"""

def BlackScholes(stock, strike, sig, time, riskfree=0.03, dividend=0, put=False):
    d1 = (numpy.log(stock/strike) + (riskfree - dividend + (sig**2)/2)*time)/(sig*numpy.sqrt(time))
    d2 = d1 - sig*numpy.sqrt(time)
    if put:
        return strike*helper.phi(-d2)*numpy.exp(-riskfree*time) - numpy.exp(-dividend*time)*stock*helper.phi(-d1)
    return numpy.exp(-dividend*time)*stock*helper.phi(d1) - strike*helper.phi(d2)*numpy.exp(-riskfree*time)

def binomial(n, stock, strike, sigma, time, riskfree=0.03, put=False, American=False):
    dt = time/n
    up_rate = numpy.exp(sigma*numpy.sqrt(dt))
    down_rate = 1/up_rate
    p = (numpy.exp((riskfree)*dt) - down_rate)/(up_rate - down_rate)
    
    list = []
    for i in range(0,n+1):
        price = stock*(up_rate**(2*i-n))
        if put:
            if price < strike:
                list.append(strike - price)
            else:
                list.append(0.0) 
        else:
            if price > strike:
                list.append(price - strike)
            else:
                list.append(0.0) 
    for j in range(0,n):
        for i in range(0,n-j):
            if American == False:
                list[i] = (list[i]*(1-p) + list[i+1]*(p))*numpy.exp(-riskfree*dt)
            else:
                value = (list[i]*(1-p) + list[i+1]*(p))*numpy.exp(-riskfree*dt)
                exercise = stock*(up_rate**(2*i-n+j+1)) - price   
                if value > exercise:
                    list[i] = (list[i]*(1-p) + list[i+1]*(p))*numpy.exp(-riskfree*dt)
                else:
                    list[i] = exercise
    return list[0]

def TheGreeks(stock, strike, sigma, time, riskfree=0.03, put=False):
    
    d1 = (numpy.log(stock/strike) + (riskfree + (sigma**2)/2)*time)/(sigma*numpy.sqrt(time))
    d2 = d1 - sigma*numpy.sqrt(time)
    Nprime1 = helper.phiprime(d1)
    
    gamma = Nprime1/(stock*sigma*math.sqrt(time))
    vega = stock*Nprime1*math.sqrt(time)
    if put:
        delta = helper.phi(d1) - 1
        theta = -(stock*Nprime1*sigma/(2*math.sqrt(time))) + riskfree*strike*numpy.exp(-riskfree*time)*helper.phi(-d2)
        rho = -strike*time*numpy.exp(-riskfree*time)*helper.phi(-d2)
    else:
        delta = helper.phi(d1)
        theta = -(stock*Nprime1*sigma/(2*math.sqrt(time))) - riskfree*strike*numpy.exp(-riskfree*time)*helper.phi(d2)
        rho = strike*time*numpy.exp(-riskfree*time)*helper.phi(d2)
    return [delta, gamma, vega/100, theta/365, rho/10000]

def MonteCarlo(runs, n, stock, strike, sigma, sig_mu, sig_sig, time, riskfree=0.03):
    simulations = []
    dt = time/n
    discount = numpy.exp(-riskfree*time)
    for r in range(0,runs):
        price = stock
        sig = sigma
        for n in range(0,n):
            sig = sig*numpy.exp(random.gauss(sig_mu,sig_sig)*dt)
            up_rate = numpy.exp(sig*numpy.sqrt(dt))
            down_rate = 1/up_rate
            p = (numpy.exp((riskfree)*dt) - down_rate)/(up_rate - down_rate)
            if random.random() <= p:
                price = price*up_rate
            else:
                price = price*down_rate
        if price > strike:
            simulations.append((price-strike))
        else:
            simulations.append(0)
    return sum(simulations)*discount/len(simulations)