import math
import datetime
from curtin_options_tools import fetch
from curtin_options_tools import helper
from curtin_options_tools import options

def historicalV(symbol, start, end):
    def std(data):
        mean = 0
        for i in data:
            mean = i['returns'] + mean
        mean = mean/len(data)
        sum = 0
        for i in data:
            sum = sum + (i['returns'] - mean)**2
        return math.sqrt(sum/(len(data)-1))
    begin = helper.str2datetime(start)
    start = begin - datetime.timedelta(36)
    
    data = fetch.fetchStockData(symbol, start, end)
    for i in range(1,len(data)):
        data[i]['returns'] = math.log(data[i]['close']/data[i-1]['close'])
    data.remove(data[0])
    histV = {}
    
    for i in range(len(data)):
        day1 = helper.str2datetime(data[i]['date'])
        day2 = helper.str2datetime(data[i+1]['date'])
        if abs((begin - day1).days) < abs((begin - day2).days):
            position = i
            break

    del data[0:position-20]
    for i in range(20,len(data)):
        histV[data[i]['date']] = dict(close=data[i]['close'],volatility=std(data[i-20:i])*math.sqrt(252))
        
    return histV

def impliedV(symbol, data, interp=28):
    for i in data.items():
        today = helper.str2datetime(i[0])
        
        rolling = today + datetime.timedelta(interp)
        
        fridays = []
        fridays.append(rolling + datetime.timedelta((4-today.weekday()) % 7))
        fridays.append(fridays[0] - datetime.timedelta(7))

        strike = 2.5 * round(i[1]['close']/2.5)
        
        def getSigma():
            try:
                i[1][key] = options.getSigma(i[1]['close'], strike, price['close'], days/365)
            except options.SigmaZeroError:
                if o == 1:
                    i[1][key] = i[1]['IV0']
                else:
                    i[1][key] = None

        for o in range(0,2):
            key = 'IV' + str(o)
            days = (fridays[o] - today).days
            print("Day: " + i[0] + ", Expiration: " + str(fridays[o]))
            try:
                price = fetch.fetchOptionData(symbol, str(fridays[o]), today, today, strike)
            except AttributeError:
                try:
                    price = fetch.fetchOptionData(symbol, str(fridays[o] - datetime.timedelta(1)), today, today, strike)
                except AttributeError:
                    try:
                        strike = 5 * round(i[1]['close']/5)
                        price = fetch.fetchOptionData(symbol, str(fridays[o]), today, today, strike)
                    except:
                        if o == 1:
                            print('Option expiration too far out.')
                            i[1][key] = i[1]['IV0']
                        else:
                            print("Data does not exist.")
                            i[1][key] = None
                    else:
                        getSigma()
                else:
                    getSigma()
            else:
                getSigma()
        
        try:
            d1 = abs(interp**0.5 - (fridays[0] - today).days**0.5)
            d2 = abs(interp**0.5 - (fridays[1] - today).days**0.5)
            delta = d1 + d2
            i[1]['weighted'] = ((i[1]['IV0'])*(delta - d1)/delta) + ((i[1]['IV1'])*(d1)/delta)
        except TypeError:
            if (i[1]['IV1'] != None):
                i[1]['weighted'] = i[1]['IV1']
            elif (i[1]['IV0'] != None):
                i[1]['weighted'] = i[1]['IV0']
            else:
                i[1]['weighted'] = None
                
        try:
            i[1]['ratio'] = i[1]['weighted']/i[1]['volatility']
        except TypeError:
            i[1]['ratio'] = None
            
    return data

def getVol(stock, start, end, interp=28):
    return impliedV(stock, historicalV(stock, start, end), interp)
    
    
    
    
    